// Delegates Demonstration by Stoyan Iordanov

using System;

namespace DelegatesDemo2
{
	public delegate int IntFunction(int i);

	class MathClass
	{
		private class FunctionCombiner
		{
			IntFunction f1;
			IntFunction f2;

			public FunctionCombiner(IntFunction f1, IntFunction f2)
			{
				this.f1 = f1;
				this.f2 = f2;
			}

			private int CombinedFunction(int x)
			{
				return f2(f1(x));
			}

			private int AddedFunction(int x)
			{
				return f1(x) + f2(x);
			}

			public IntFunction Combiner
			{
				get
				{
					return new IntFunction(CombinedFunction);
				}
			}

			public IntFunction Adder
			{
				get
				{
					return new IntFunction(AddedFunction);
				}
			}
		}

		//  ,     
		public static IntFunction SumFunctions(IntFunction f1, IntFunction f2)
		{
			FunctionCombiner combinerObject = new FunctionCombiner(f1, f2);
			return combinerObject.Adder;
		}

		//  ,     
		public static IntFunction CombineFunctions(IntFunction f1, IntFunction f2)
		{
			FunctionCombiner combinerObject = new FunctionCombiner(f1, f2);
			return combinerObject.Combiner;
		}
	}

	class MainClass
	{
		private static int Power2(int x)
		{
			return x * x;
		}

		private static int Add3(int x)
		{
			return x + 3;
		}

		static void Main(string[] args)
		{
			Console.Write("Enter x: ");
			int x = int.Parse(Console.ReadLine());

			IntFunction power2Function = new IntFunction(Power2); // f(x) = x*x
			IntFunction add3Function = new IntFunction(Add3);     // f(x) = x + 3

			Console.WriteLine("f(x) = x^2 = {0}", power2Function(x));
			Console.WriteLine("f(x) = x + 3 = {0}", add3Function(x));

			// f(x) = x^2 + (x + 3)
			IntFunction summedFunctions =
				MathClass.SumFunctions(power2Function, add3Function);

			// f(x) = (x + 3)^2
			IntFunction combinedFunctions =
				MathClass.CombineFunctions(add3Function, power2Function);

			Console.WriteLine("f(x) = x^2 + (x + 3) = {0}", summedFunctions(x));
			Console.WriteLine("f(x) = (x + 3)^2 = {0}", combinedFunctions(x));
		}
	}
}
