// Events Demonstration by Stoyan Iordanov

using System;

namespace EventsDemo
{
	public class AlarmSystem
	{
		public delegate void AlarmSwitchHandler(string username);

		public event AlarmSwitchHandler AlarmActivated;
		public event AlarmSwitchHandler AlarmDeactivated;

		public void Activate(string username)
		{
			Console.WriteLine("Activating alarm...");

			if(AlarmActivated != null)
			{
				AlarmActivated(username);
			}
		}

		public void Deactivate(string username)
		{
			Console.WriteLine("Deactivating alarm...");
			
			if(AlarmDeactivated != null)
			{
				AlarmDeactivated(username);
			}
		}
	}

	public class AlarmLogger
	{
		public AlarmLogger(AlarmSystem alarm)
		{
			alarm.AlarmActivated +=
				new AlarmSystem.AlarmSwitchHandler(LogActivation);

			alarm.AlarmDeactivated +=
				new AlarmSystem.AlarmSwitchHandler(LogDeactivation);
		}

		private void LogActivation(string username)
		{
			Console.WriteLine("Alarm activated by {0}", username);
		}

		private void LogDeactivation(string username)
		{
			Console.WriteLine("Alarm deactivated by {0}", username);
		}
	}

	public class AutomaticLocker
	{
		private string doorName;

		public AutomaticLocker(string doorName)
		{
			this.doorName = doorName;
		}

		public void LockDoor(string username)
		{
			Console.WriteLine("{0} locked by {1}", doorName, username);
		}

		public void UnlockDoor(string username)
		{
			Console.WriteLine("{0} unlocked by {1}", doorName, username);
		}
	}

	class MainClass
	{
		static void Main(string[] args)
		{
			//    
			AlarmSystem myAlarm = new AlarmSystem();
			
			//  ,      
			new AlarmLogger(myAlarm);

			//   ,    
			AutomaticLocker myLocker = new AutomaticLocker("Front door");

			myAlarm.AlarmActivated +=
				new AlarmSystem.AlarmSwitchHandler(myLocker.LockDoor);

			myAlarm.AlarmDeactivated +=
				new AlarmSystem.AlarmSwitchHandler(myLocker.UnlockDoor);

			//    
			myAlarm.Activate("User1");
			myAlarm.Deactivate("User2");
		}
	}
}
